import { useCallback, useEffect, useState } from "preact/hooks";
import { DataRow, DataRowSearchOptions } from "../api/DataRow";
import style from './index.module.scss';
import { Posts } from "../components/Posts";

import InfiniteScroller from 'react-infinite-scroller';
import { DraggablePostsList } from "../components/DraggablePostsList";
import { DataRowSelection } from "../api/DataRowSelection";
import { SearchBar } from "../components/display/SearchBar";
import delay from "delay";
import { NewPost } from "../components/display/NewPost";

export function Index() {
    const [ data, setData ] = useState(null as null | DataRow[]);
    const [ selected, setSelected ] = useState(null as null | DataRow[]);
    const dataRowSelection = new DataRowSelection(selected ?? []);

    const [ hasMore, setHasMore ] = useState(true);
    const [ fetching, setFetching ] = useState(false);

    const [ filter, setFilter ] = useState(null as null | string);

    useEffect(() => {
        (async () => {
            setSelected(await dataRowSelection.get());
        })();
    }, []);

    const fetchMore = useCallback(
        async function() {
            if (fetching) return;

            setFetching(true);
            
            const query = {
                page: data ? Math.floor(data.length / 20) : 0,
            } as DataRowSearchOptions;
            if (filter) query.search = `%${filter}%`;

            const newData = await DataRow.find(query);
            setData([ ...data ?? [], ...newData ]);
            setHasMore(newData.length === 20);
            setFetching(false);
        },
        [ fetching, hasMore ]
    );

    async function saveSelection(rows: DataRow[]) {
        setSelected(rows);
        dataRowSelection.set(rows);
        await dataRowSelection.save();
    }

    function select(row: DataRow) {
        console.log(row);
        if (data) {
            if (data.indexOf(row) == -1) throw new Error('Selected element that does not exist');
            saveSelection([ ...selected ?? [], row ])
            setData(data)
        }
    }

    async function reload() {
        setFetching(true);
        
        setHasMore(true);
        setData(null);
        await delay(0); // stupid thing won't work w/o delay

        setFetching(false);
    }

    async function search(query: string) {
        setFilter(query);
        reload();
    }

    async function clearSelection() {
        setSelected(dataRowSelection.set([]));
        await dataRowSelection.save();
    }

    return (
        <div className={style.index}>
            {
                selected
                    ?
                        <div className={style.selection}>
                            <h2>Selection</h2>
                            <button onClick={clearSelection}>clear selection</button>
                            <DraggablePostsList rows={selected} setRows={saveSelection} />
                        </div>
                    : null
            }
            
            <SearchBar onSearch={search} placeholder="filter" />
            <NewPost onCreate={reload} />
            
            <InfiniteScroller
                pageStart={0}
                loadMore={fetchMore}
                hasMore={hasMore}
                loader={<pre>loading . . .</pre>}
                useWindow={true}
                >
                {
                    data
                        ? <Posts rows={data.filter(x => selected ? selected?.indexOf(x) === -1 : true)} onSelectChange={select} />
                        : <pre>no data!</pre>
                }
            </InfiniteScroller>
        </div>
    )
}